package java.awt;

import java.util.Arrays;
import java.util.Hashtable;
import java.io.Serializable;

public class GridBagLayout implements LayoutManager2, Serializable
{
    static final int EMPIRICMULTIPLIER = 2;
    protected static final int MAXGRIDSIZE = 512;
    protected static final int MINSIZE = 1;
    protected static final int PREFERREDSIZE = 2;
    protected Hashtable<Component, GridBagConstraints> comptable;
    protected GridBagConstraints defaultConstraints;
    protected GridBagLayoutInfo layoutInfo;
    public int[] columnWidths;
    public int[] rowHeights;
    public double[] columnWeights;
    public double[] rowWeights;
    private Component componentAdjusting;
    transient boolean rightToLeft;
    static final long serialVersionUID = 8838754796412211005L;
    
    public GridBagLayout() {
        this.rightToLeft = false;
        this.comptable = new Hashtable();
        this.defaultConstraints = new GridBagConstraints();
    }
    
    public void setConstraints(Component key, GridBagConstraints gridBagConstraints) {
        this.comptable.put(key, (GridBagConstraints)gridBagConstraints.clone());
    }
    
    public GridBagConstraints getConstraints(Component component) {
        GridBagConstraints gridBagConstraints = (GridBagConstraints)this.comptable.get(component);
        if (gridBagConstraints == null) {
            this.setConstraints(component, this.defaultConstraints);
            gridBagConstraints = (GridBagConstraints)this.comptable.get(component);
        }
        return (GridBagConstraints)gridBagConstraints.clone();
    }
    
    protected GridBagConstraints lookupConstraints(Component component) {
        GridBagConstraints gridBagConstraints = (GridBagConstraints)this.comptable.get(component);
        if (gridBagConstraints == null) {
            this.setConstraints(component, this.defaultConstraints);
            gridBagConstraints = (GridBagConstraints)this.comptable.get(component);
            return gridBagConstraints;
        }
        return gridBagConstraints;
    }
    
    private void removeConstraints(Component key) {
        this.comptable.remove(key);
    }
    
    public Point getLayoutOrigin() {
        Point point = new Point(0, 0);
        if (this.layoutInfo != null) {
            point.x = this.layoutInfo.startx;
            point.y = this.layoutInfo.starty;
            return point;
        }
        return point;
    }
    
    public int[][] getLayoutDimensions() {
        if (this.layoutInfo == null) {
            return new int[2][0];
        }
        int[][] array = { new int[this.layoutInfo.width], new int[this.layoutInfo.height] };
        System.arraycopy(this.layoutInfo.minWidth, 0, array[0], 0, this.layoutInfo.width);
        System.arraycopy(this.layoutInfo.minHeight, 0, array[1], 0, this.layoutInfo.height);
        return array;
    }
    
    public double[][] getLayoutWeights() {
        if (this.layoutInfo == null) {
            return new double[2][0];
        }
        double[][] array = { new double[this.layoutInfo.width], new double[this.layoutInfo.height] };
        System.arraycopy(this.layoutInfo.weightX, 0, array[0], 0, this.layoutInfo.width);
        System.arraycopy(this.layoutInfo.weightY, 0, array[1], 0, this.layoutInfo.height);
        return array;
    }
    
    public Point location(int n, int n2) {
        Point point = new Point(0, 0);
        if (this.layoutInfo == null) {
            return point;
        }
        int startx = this.layoutInfo.startx;
        int i;
        if (!this.rightToLeft) {
            i = 0;
            while (i < this.layoutInfo.width) {
                startx += this.layoutInfo.minWidth[i];
                if (startx > n) {
                    break;
                }
                ++i;
            }
        }
        else {
            for (i = this.layoutInfo.width - 1; i >= 0 && startx <= n; startx += this.layoutInfo.minWidth[i], --i) {}
            ++i;
        }
        point.x = i;
        int starty = this.layoutInfo.starty;
        int j = 0;
        while (j < this.layoutInfo.height) {
            starty += this.layoutInfo.minHeight[j];
            if (starty > n2) {
                break;
            }
            ++j;
        }
        point.y = j;
        return point;
    }
    
    @Override
    public void addLayoutComponent(String s, Component component) {
    }
    
    @Override
    public void addLayoutComponent(Component component, Object o) {
        if ((o instanceof GridBagConstraints)) {
            this.setConstraints(component, (GridBagConstraints)o);
        }
        else if (o != null) {
            throw new IllegalArgumentException("cannot add to layout: constraints must be a GridBagConstraint");
        }
    }
    
    @Override
    public void removeLayoutComponent(Component component) {
        this.removeConstraints(component);
    }
    
    @Override
    public Dimension preferredLayoutSize(Container container) {
        return this.getMinSize(container, this.getLayoutInfo(container, 2));
    }
    
    @Override
    public Dimension minimumLayoutSize(Container container) {
        return this.getMinSize(container, this.getLayoutInfo(container, 1));
    }
    
    @Override
    public Dimension maximumLayoutSize(Container container) {
        return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
    }
    
    @Override
    public float getLayoutAlignmentX(Container container) {
        return 0.5f;
    }
    
    @Override
    public float getLayoutAlignmentY(Container container) {
        return 0.5f;
    }
    
    @Override
    public void invalidateLayout(Container container) {
    }
    
    @Override
    public void layoutContainer(Container container) {
        this.arrangeGrid(container);
    }
    
    @Override
    public String toString() {
        return this.getClass().getName();
    }
    
    protected GridBagLayoutInfo getLayoutInfo(Container container, int n) {
        return this.GetLayoutInfo(container, n);
    }
    
    private long[] preInitMaximumArraySizes(Container container) {
        Component[] components = container.getComponents();
        int max = 0;
        int max2 = 0;
        long[] array = new long[2];
        for (int i = 0; i < components.length; ++i) {
            Component component = components[i];
            if (component.isVisible()) {
                GridBagConstraints lookupConstraints = this.lookupConstraints(component);
                int gridx = lookupConstraints.gridx;
                int gridy = lookupConstraints.gridy;
                int gridwidth = lookupConstraints.gridwidth;
                int gridheight = lookupConstraints.gridheight;
                if (gridx < 0) {
                    gridx = ++max2;
                }
                if (gridy < 0) {
                    gridy = ++max;
                }
                if (gridwidth <= 0) {
                    gridwidth = 1;
                }
                if (gridheight <= 0) {
                    gridheight = 1;
                }
                max = Math.max(gridy + gridheight, max);
                max2 = Math.max(gridx + gridwidth, max2);
            }
        }
        array[0] = (long)max;
        array[1] = (long)max2;
        return array;
    }
    
    protected GridBagLayoutInfo GetLayoutInfo(Container container, int n) {
        synchronized (container.getTreeLock()) {
            Component[] components = container.getComponents();
            int length2;
            int length = length2 = 0;
            int n3;
            int n2 = n3 = -1;
            long[] preInitMaximumArraySizes = this.preInitMaximumArraySizes(container);
            int max = (2L * preInitMaximumArraySizes[0] > 2147483647L) ? Integer.MAX_VALUE : (2 * (int)preInitMaximumArraySizes[0]);
            int max2 = (2L * preInitMaximumArraySizes[1] > 2147483647L) ? Integer.MAX_VALUE : (2 * (int)preInitMaximumArraySizes[1]);
            if (this.rowHeights != null) {
                max = Math.max(max, this.rowHeights.length);
            }
            if (this.columnWidths != null) {
                max2 = Math.max(max2, this.columnWidths.length);
            }
            int[] a = new int[max];
            int[] a2 = new int[max2];
            boolean n4 = false;
            for (int i = 0; i < components.length; ++i) {
                Component component = components[i];
                if (component.isVisible()) {
                    GridBagConstraints lookupConstraints = this.lookupConstraints(component);
                    int j = lookupConstraints.gridx;
                    int k = lookupConstraints.gridy;
                    int gridwidth = lookupConstraints.gridwidth;
                    if (gridwidth <= 0) {
                        gridwidth = 1;
                    }
                    int gridheight = lookupConstraints.gridheight;
                    if (gridheight <= 0) {
                        gridheight = 1;
                    }
                    if (j < 0 && k < 0) {
                        if (n3 >= 0) {
                            k = n3;
                        }
                        else if (n2 >= 0) {
                            j = n2;
                        }
                        else {
                            k = 0;
                        }
                    }
                    if (j < 0) {
                        int max4 = 0;
                        for (; k < k + gridheight; ++k) {
                            max4 = Math.max(max4, a[k]);
                        }
                        j = max4 - j - 1;
                        if (j < 0) {
                            j = 0;
                        }
                    }
                    else if (k < 0) {
                        int max3 = 0;
                        for (; j < j + gridwidth; ++j) {
                            max3 = Math.max(max3, a2[j]);
                        }
                        k = max3 - k - 1;
                        if (k < 0) {
                            k = 0;
                        }
                    }
                    int n5 = j + gridwidth;
                    if (length2 < n5) {
                        length2 = n5;
                    }
                    int n6 = k + gridheight;
                    if (length < n6) {
                        length = n6;
                    }
                    for (; j < j + gridwidth; ++j) {
                        a2[j] = n6;
                    }
                    for (; k < k + gridheight; ++k) {
                        a[k] = n5;
                    }
                    Dimension dimension;
                    if (n == 2) {
                        dimension = component.getPreferredSize();
                    }
                    else {
                        dimension = component.getMinimumSize();
                    }
                    lookupConstraints.minWidth = dimension.width;
                    lookupConstraints.minHeight = dimension.height;
                    if (this.calculateBaseline(component, lookupConstraints, dimension)) {
                        n4 = true;
                    }
                    if (lookupConstraints.gridheight == 0 && lookupConstraints.gridwidth == 0) {
                        n2 = (n3 = -1);
                    }
                    if (lookupConstraints.gridheight == 0 && n3 < 0) {
                        n2 = j + gridwidth;
                    }
                    else if (lookupConstraints.gridwidth == 0 && n2 < 0) {
                        n3 = k + gridheight;
                    }
                }
            }
            if (this.columnWidths != null && length2 < this.columnWidths.length) {
                length2 = this.columnWidths.length;
            }
            if (this.rowHeights != null && length < this.rowHeights.length) {
                length = this.rowHeights.length;
            }
            GridBagLayoutInfo o = new GridBagLayoutInfo(length2, length);
            int n8;
            int n7 = n8 = -1;
            Arrays.fill(a, 0);
            Arrays.fill(a2, 0);
            int[] array = null;
            int[] array2 = null;
            short[] array3 = null;
            if (!(n4 == 0)) {
                array = (o.maxAscent = new int[length]);
                array2 = (o.maxDescent = new int[length]);
                array3 = (o.baselineType = new short[length]);
                o.hasBaseline = true;
            }
            for (int l = 0; l < components.length; ++l) {
                Component component2 = components[l];
                if (component2.isVisible()) {
                    GridBagConstraints lookupConstraints2 = this.lookupConstraints(component2);
                    int gridx = lookupConstraints2.gridx;
                    int gridy = lookupConstraints2.gridy;
                    int gridwidth2 = lookupConstraints2.gridwidth;
                    int gridheight2 = lookupConstraints2.gridheight;
                    if (gridx < 0 && gridy < 0) {
                        if (n8 >= 0) {
                            gridy = n8;
                        }
                        else if (n7 >= 0) {
                            gridx = n7;
                        }
                        else {
                            gridy = 0;
                        }
                    }
                    if (gridx < 0) {
                        if (gridheight2 <= 0) {
                            gridheight2 += o.height - gridy;
                            if (gridheight2 < 1) {
                                gridheight2 = 1;
                            }
                        }
                        int max6 = 0;
                        for (; gridy < gridy + gridheight2; ++gridy) {
                            max6 = Math.max(max6, a[gridy]);
                        }
                        gridx = max6 - gridx - 1;
                        if (gridx < 0) {
                            gridx = 0;
                        }
                    }
                    else if (gridy < 0) {
                        if (gridwidth2 <= 0) {
                            gridwidth2 += o.width - gridx;
                            if (gridwidth2 < 1) {
                                gridwidth2 = 1;
                            }
                        }
                        int max5 = 0;
                        for (; gridx < gridx + gridwidth2; ++gridx) {
                            max5 = Math.max(max5, a2[gridx]);
                        }
                        gridy = max5 - gridy - 1;
                        if (gridy < 0) {
                            gridy = 0;
                        }
                    }
                    if (gridwidth2 <= 0) {
                        gridwidth2 += o.width - gridx;
                        if (gridwidth2 < 1) {
                            gridwidth2 = 1;
                        }
                    }
                    if (gridheight2 <= 0) {
                        gridheight2 += o.height - gridy;
                        if (gridheight2 < 1) {
                            gridheight2 = 1;
                        }
                    }
                    int n9 = gridx + gridwidth2;
                    int n10 = gridy + gridheight2;
                    for (; gridx < gridx + gridwidth2; ++gridx) {
                        a2[gridx] = n10;
                    }
                    for (; gridy < gridy + gridheight2; ++gridy) {
                        a[gridy] = n9;
                    }
                    if (lookupConstraints2.gridheight == 0 && lookupConstraints2.gridwidth == 0) {
                        n7 = (n8 = -1);
                    }
                    if (lookupConstraints2.gridheight == 0 && n8 < 0) {
                        n7 = gridx + gridwidth2;
                    }
                    else if (lookupConstraints2.gridwidth == 0 && n7 < 0) {
                        n8 = gridy + gridheight2;
                    }
                    lookupConstraints2.tempX = gridx;
                    lookupConstraints2.tempY = gridy;
                    lookupConstraints2.tempWidth = gridwidth2;
                    lookupConstraints2.tempHeight = gridheight2;
                    int anchor = lookupConstraints2.anchor;
                    if (!(n4 == 0)) {
                        switch (anchor) {
                            case 256:
                            case 512:
                            case 768: {
                                if (lookupConstraints2.ascent >= 0) {
                                    if (gridheight2 == 1) {
                                        array[gridy] = Math.max(array[gridy], lookupConstraints2.ascent);
                                        array2[gridy] = Math.max(array2[gridy], lookupConstraints2.descent);
                                    }
                                    else if (lookupConstraints2.baselineResizeBehavior == Component.BaselineResizeBehavior.CONSTANT_DESCENT) {
                                        array2[gridy + gridheight2 - 1] = Math.max(array2[gridy + gridheight2 - 1], lookupConstraints2.descent);
                                    }
                                    else {
                                        array[gridy] = Math.max(array[gridy], lookupConstraints2.ascent);
                                    }
                                    if (lookupConstraints2.baselineResizeBehavior == Component.BaselineResizeBehavior.CONSTANT_DESCENT) {
                                        int n12 = gridy + gridheight2 - 1;
                                        array3[n12] |= 1 << lookupConstraints2.baselineResizeBehavior.ordinal();
                                        break;
                                    }
                                    int n11 = gridy;
                                    array3[n11] |= 1 << lookupConstraints2.baselineResizeBehavior.ordinal();
                                    break;
                                }
                                else {
                                    break;
                                }
                            }
                            case 1024:
                            case 1280:
                            case 1536: {
                                array[gridy] = Math.max(array[gridy], lookupConstraints2.minHeight + lookupConstraints2.insets.top + lookupConstraints2.ipady);
                                array2[gridy] = Math.max(array2[gridy], lookupConstraints2.insets.bottom);
                                break;
                            }
                            case 1792:
                            case 2048:
                            case 2304: {
                                array2[gridy] = Math.max(array2[gridy], lookupConstraints2.minHeight + lookupConstraints2.insets.bottom + lookupConstraints2.ipady);
                                array[gridy] = Math.max(array[gridy], lookupConstraints2.insets.top);
                                break;
                            }
                        }
                    }
                }
            }
            o.weightX = new double[max2];
            o.weightY = new double[max];
            o.minWidth = new int[max2];
            o.minHeight = new int[max];
            if (this.columnWidths != null) {
                System.arraycopy(this.columnWidths, 0, o.minWidth, 0, this.columnWidths.length);
            }
            if (this.rowHeights != null) {
                System.arraycopy(this.rowHeights, 0, o.minHeight, 0, this.rowHeights.length);
            }
            if (this.columnWeights != null) {
                System.arraycopy(this.columnWeights, 0, o.weightX, 0, Math.min(o.weightX.length, this.columnWeights.length));
            }
            if (this.rowWeights != null) {
                System.arraycopy(this.rowWeights, 0, o.weightY, 0, Math.min(o.weightY.length, this.rowWeights.length));
            }
            for (int n13 = Integer.MAX_VALUE, n14 = 1; n14 != Integer.MAX_VALUE; n14 = n13, n13 = Integer.MAX_VALUE) {
                for (int n15 = 0; n15 < components.length; ++n15) {
                    Component component3 = components[n15];
                    if (component3.isVisible()) {
                        GridBagConstraints lookupConstraints3 = this.lookupConstraints(component3);
                        if (lookupConstraints3.tempWidth == n14) {
                            int n16 = lookupConstraints3.tempX + lookupConstraints3.tempWidth;
                            double weightx = lookupConstraints3.weightx;
                            for (int tempX = lookupConstraints3.tempX; tempX < n16; ++tempX) {
                                weightx -= o.weightX[tempX];
                            }
                            if (weightx > 0.0) {
                                double n17 = 0.0;
                                for (int tempX2 = lookupConstraints3.tempX; tempX2 < n16; ++tempX2) {
                                    n17 += o.weightX[tempX2];
                                }
                                double n18;
                                for (int tempX3 = lookupConstraints3.tempX; n17 > 0.0 && tempX3 < n16; n17 -= n18, ++tempX3) {
                                    n18 = o.weightX[tempX3];
                                    double n19 = n18 * weightx / n17;
                                    double[] weightX = o.weightX;
                                    int n20 = tempX3;
                                    weightX[n20] += n19;
                                    weightx -= n19;
                                }
                                double[] weightX2 = o.weightX;
                                int n21 = n16 - 1;
                                weightX2[n21] += weightx;
                            }
                            int n22 = lookupConstraints3.minWidth + lookupConstraints3.ipadx + lookupConstraints3.insets.left + lookupConstraints3.insets.right;
                            for (int tempX4 = lookupConstraints3.tempX; tempX4 < n16; ++tempX4) {
                                n22 -= o.minWidth[tempX4];
                            }
                            if (n22 > 0) {
                                double n23 = 0.0;
                                for (int tempX5 = lookupConstraints3.tempX; tempX5 < n16; ++tempX5) {
                                    n23 += o.weightX[tempX5];
                                }
                                double n24;
                                for (int tempX6 = lookupConstraints3.tempX; n23 > 0.0 && tempX6 < n16; n23 -= n24, ++tempX6) {
                                    n24 = o.weightX[tempX6];
                                    int n25 = (int)(n24 * (double)n22 / n23);
                                    int[] minWidth = o.minWidth;
                                    int n26 = tempX6;
                                    minWidth[n26] += n25;
                                    n22 -= n25;
                                }
                                int[] minWidth2 = o.minWidth;
                                int n27 = n16 - 1;
                                minWidth2[n27] += n22;
                            }
                        }
                        else if (lookupConstraints3.tempWidth > n14 && lookupConstraints3.tempWidth < n13) {
                            n13 = lookupConstraints3.tempWidth;
                        }
                        if (lookupConstraints3.tempHeight == n14) {
                            int n28 = lookupConstraints3.tempY + lookupConstraints3.tempHeight;
                            double weighty = lookupConstraints3.weighty;
                            for (int tempY = lookupConstraints3.tempY; tempY < n28; ++tempY) {
                                weighty -= o.weightY[tempY];
                            }
                            if (weighty > 0.0) {
                                double n29 = 0.0;
                                for (int tempY2 = lookupConstraints3.tempY; tempY2 < n28; ++tempY2) {
                                    n29 += o.weightY[tempY2];
                                }
                                double n30;
                                for (int tempY3 = lookupConstraints3.tempY; n29 > 0.0 && tempY3 < n28; n29 -= n30, ++tempY3) {
                                    n30 = o.weightY[tempY3];
                                    double n31 = n30 * weighty / n29;
                                    double[] weightY = o.weightY;
                                    int n32 = tempY3;
                                    weightY[n32] += n31;
                                    weighty -= n31;
                                }
                                double[] weightY2 = o.weightY;
                                int n33 = n28 - 1;
                                weightY2[n33] += weighty;
                            }
                            int n34 = -1;
                            if (!(n4 == 0)) {
                                switch (lookupConstraints3.anchor) {
                                    case 256:
                                    case 512:
                                    case 768: {
                                        if (lookupConstraints3.ascent >= 0) {
                                            if (lookupConstraints3.tempHeight == 1) {
                                                n34 = array[lookupConstraints3.tempY] + array2[lookupConstraints3.tempY];
                                                break;
                                            }
                                            if (lookupConstraints3.baselineResizeBehavior != Component.BaselineResizeBehavior.CONSTANT_DESCENT) {
                                                n34 = array[lookupConstraints3.tempY] + lookupConstraints3.descent;
                                                break;
                                            }
                                            n34 = lookupConstraints3.ascent + array2[lookupConstraints3.tempY + lookupConstraints3.tempHeight - 1];
                                            break;
                                        }
                                        else {
                                            break;
                                        }
                                    }
                                    case 1024:
                                    case 1280:
                                    case 1536: {
                                        n34 = lookupConstraints3.insets.top + lookupConstraints3.minHeight + lookupConstraints3.ipady + array2[lookupConstraints3.tempY];
                                        break;
                                    }
                                    case 1792:
                                    case 2048:
                                    case 2304: {
                                        n34 = array[lookupConstraints3.tempY] + lookupConstraints3.minHeight + lookupConstraints3.insets.bottom + lookupConstraints3.ipady;
                                        break;
                                    }
                                }
                            }
                            if (n34 == -1) {
                                n34 = lookupConstraints3.minHeight + lookupConstraints3.ipady + lookupConstraints3.insets.top + lookupConstraints3.insets.bottom;
                            }
                            for (int tempY4 = lookupConstraints3.tempY; tempY4 < n28; ++tempY4) {
                                n34 -= o.minHeight[tempY4];
                            }
                            if (n34 > 0) {
                                double n35 = 0.0;
                                for (int tempY5 = lookupConstraints3.tempY; tempY5 < n28; ++tempY5) {
                                    n35 += o.weightY[tempY5];
                                }
                                double n36;
                                for (int tempY6 = lookupConstraints3.tempY; n35 > 0.0 && tempY6 < n28; n35 -= n36, ++tempY6) {
                                    n36 = o.weightY[tempY6];
                                    int n37 = (int)(n36 * (double)n34 / n35);
                                    int[] minHeight = o.minHeight;
                                    int n38 = tempY6;
                                    minHeight[n38] += n37;
                                    n34 -= n37;
                                }
                                int[] minHeight2 = o.minHeight;
                                int n39 = n28 - 1;
                                minHeight2[n39] += n34;
                            }
                        }
                        else if (lookupConstraints3.tempHeight > n14 && lookupConstraints3.tempHeight < n13) {
                            n13 = lookupConstraints3.tempHeight;
                        }
                    }
                }
            }
            return o;
        }
    }
    
    private boolean calculateBaseline(Component component, GridBagConstraints gridBagConstraints, Dimension dimension) {
        int anchor = gridBagConstraints.anchor;
        if (anchor == 256 || anchor == 512 || anchor == 768) {
            int n = dimension.width + gridBagConstraints.ipadx;
            int n2 = dimension.height + gridBagConstraints.ipady;
            gridBagConstraints.ascent = component.getBaseline(n, n2);
            if (gridBagConstraints.ascent >= 0) {
                int ascent = gridBagConstraints.ascent;
                gridBagConstraints.descent = n2 - gridBagConstraints.ascent + gridBagConstraints.insets.bottom;
                gridBagConstraints.ascent += gridBagConstraints.insets.top;
                gridBagConstraints.baselineResizeBehavior = component.getBaselineResizeBehavior();
                gridBagConstraints.centerPadding = 0;
                if (gridBagConstraints.baselineResizeBehavior == Component.BaselineResizeBehavior.CENTER_OFFSET) {
                    int baseline = component.getBaseline(n, n2 + 1);
                    gridBagConstraints.centerOffset = ascent - n2 / 2;
                    if (n2 % 2 == 0) {
                        if (ascent != baseline) {
                            gridBagConstraints.centerPadding = 1;
                        }
                    }
                    else if (ascent == baseline) {
                        --gridBagConstraints.centerOffset;
                        gridBagConstraints.centerPadding = 1;
                    }
                }
            }
            return true;
        }
        else {
            gridBagConstraints.ascent = -1;
            return false;
        }
    }
    
    protected void adjustForGravity(GridBagConstraints gridBagConstraints, Rectangle rectangle) {
        this.AdjustForGravity(gridBagConstraints, rectangle);
    }
    
    protected void AdjustForGravity(GridBagConstraints gridBagConstraints, Rectangle rectangle) {
        int y = rectangle.y;
        int height = rectangle.height;
        if (!this.rightToLeft) {
            rectangle.x += gridBagConstraints.insets.left;
        }
        else {
            rectangle.x -= rectangle.width - gridBagConstraints.insets.right;
        }
        rectangle.width -= gridBagConstraints.insets.left + gridBagConstraints.insets.right;
        rectangle.y += gridBagConstraints.insets.top;
        rectangle.height -= gridBagConstraints.insets.top + gridBagConstraints.insets.bottom;
        int n = 0;
        if (gridBagConstraints.fill != 2 && gridBagConstraints.fill != 1 && rectangle.width > gridBagConstraints.minWidth + gridBagConstraints.ipadx) {
            n = rectangle.width - (gridBagConstraints.minWidth + gridBagConstraints.ipadx);
            rectangle.width = gridBagConstraints.minWidth + gridBagConstraints.ipadx;
        }
        int n2 = 0;
        if (gridBagConstraints.fill != 3 && gridBagConstraints.fill != 1 && rectangle.height > gridBagConstraints.minHeight + gridBagConstraints.ipady) {
            n2 = rectangle.height - (gridBagConstraints.minHeight + gridBagConstraints.ipady);
            rectangle.height = gridBagConstraints.minHeight + gridBagConstraints.ipady;
        }
        switch (gridBagConstraints.anchor) {
            case 256: {
                rectangle.x += n / 2;
                this.alignOnBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 512: {
                if (this.rightToLeft) {
                    rectangle.x += n;
                }
                this.alignOnBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 768: {
                if (!this.rightToLeft) {
                    rectangle.x += n;
                }
                this.alignOnBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 1024: {
                rectangle.x += n / 2;
                this.alignAboveBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 1280: {
                if (this.rightToLeft) {
                    rectangle.x += n;
                }
                this.alignAboveBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 1536: {
                if (!this.rightToLeft) {
                    rectangle.x += n;
                }
                this.alignAboveBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 1792: {
                rectangle.x += n / 2;
                this.alignBelowBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 2048: {
                if (this.rightToLeft) {
                    rectangle.x += n;
                }
                this.alignBelowBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 2304: {
                if (!this.rightToLeft) {
                    rectangle.x += n;
                }
                this.alignBelowBaseline(gridBagConstraints, rectangle, y, height);
                break;
            }
            case 10: {
                rectangle.x += n / 2;
                rectangle.y += n2 / 2;
                break;
            }
            case 11:
            case 19: {
                rectangle.x += n / 2;
                break;
            }
            case 12: {
                rectangle.x += n;
                break;
            }
            case 13: {
                rectangle.x += n;
                rectangle.y += n2 / 2;
                break;
            }
            case 14: {
                rectangle.x += n;
                rectangle.y += n2;
                break;
            }
            case 15:
            case 20: {
                rectangle.x += n / 2;
                rectangle.y += n2;
                break;
            }
            case 16: {
                rectangle.y += n2;
                break;
            }
            case 17: {
                rectangle.y += n2 / 2;
                break;
            }
            case 18: {
                break;
            }
            case 21: {
                if (this.rightToLeft) {
                    rectangle.x += n;
                }
                rectangle.y += n2 / 2;
                break;
            }
            case 22: {
                if (!this.rightToLeft) {
                    rectangle.x += n;
                }
                rectangle.y += n2 / 2;
                break;
            }
            case 23: {
                if (this.rightToLeft) {
                    rectangle.x += n;
                    break;
                }
                break;
            }
            case 24: {
                if (!this.rightToLeft) {
                    rectangle.x += n;
                    break;
                }
                break;
            }
            case 25: {
                if (this.rightToLeft) {
                    rectangle.x += n;
                }
                rectangle.y += n2;
                break;
            }
            case 26: {
                if (!this.rightToLeft) {
                    rectangle.x += n;
                }
                rectangle.y += n2;
                break;
            }
            default: {
                throw new IllegalArgumentException("illegal anchor value");
            }
        }
    }
    
    private void alignOnBaseline(GridBagConstraints gridBagConstraints, Rectangle rectangle, int n, int n2) {
        if (gridBagConstraints.ascent >= 0) {
            if (gridBagConstraints.baselineResizeBehavior == Component.BaselineResizeBehavior.CONSTANT_DESCENT) {
                int n7 = n + n2 - this.layoutInfo.maxDescent[gridBagConstraints.tempY + gridBagConstraints.tempHeight - 1] + gridBagConstraints.descent - gridBagConstraints.insets.bottom;
                if (!gridBagConstraints.isVerticallyResizable()) {
                    rectangle.y = n7 - gridBagConstraints.minHeight;
                    rectangle.height = gridBagConstraints.minHeight;
                }
                else {
                    rectangle.height = n7 - n - gridBagConstraints.insets.top;
                }
            }
            else {
                int n3 = gridBagConstraints.ascent;
                int n4;
                if (this.layoutInfo.hasConstantDescent(gridBagConstraints.tempY)) {
                    n4 = n2 - this.layoutInfo.maxDescent[gridBagConstraints.tempY];
                }
                else {
                    n4 = this.layoutInfo.maxAscent[gridBagConstraints.tempY];
                }
                if (gridBagConstraints.baselineResizeBehavior == Component.BaselineResizeBehavior.OTHER) {
                    boolean n5 = false;
                    n3 = this.componentAdjusting.getBaseline(rectangle.width, rectangle.height);
                    if (n3 >= 0) {
                        n3 += gridBagConstraints.insets.top;
                    }
                    if (n3 >= 0 && n3 <= n4) {
                        if (n4 + (rectangle.height - n3 - gridBagConstraints.insets.top) <= n2 - gridBagConstraints.insets.bottom) {
                            n5 = true;
                        }
                        else if (gridBagConstraints.isVerticallyResizable()) {
                            int baseline = this.componentAdjusting.getBaseline(rectangle.width, n2 - gridBagConstraints.insets.bottom - n4 + n3);
                            if (baseline >= 0) {
                                baseline += gridBagConstraints.insets.top;
                            }
                            if (baseline >= 0 && baseline <= n3) {
                                rectangle.height = n2 - gridBagConstraints.insets.bottom - n4 + n3;
                                n3 = baseline;
                                n5 = true;
                            }
                        }
                    }
                    if (n5 == 0) {
                        n3 = gridBagConstraints.ascent;
                        rectangle.width = gridBagConstraints.minWidth;
                        rectangle.height = gridBagConstraints.minHeight;
                    }
                }
                rectangle.y = n + n4 - n3 + gridBagConstraints.insets.top;
                if (gridBagConstraints.isVerticallyResizable()) {
                    switch (gridBagConstraints.baselineResizeBehavior) {
                        case CONSTANT_ASCENT: {
                            rectangle.height = Math.max(gridBagConstraints.minHeight, n + n2 - rectangle.y - gridBagConstraints.insets.bottom);
                            break;
                        }
                        case CENTER_OFFSET: {
                            int min = Math.min(rectangle.y - n - gridBagConstraints.insets.top, n + n2 - rectangle.y - gridBagConstraints.minHeight - gridBagConstraints.insets.bottom);
                            int n6 = min + min;
                            if (n6 > 0 && (gridBagConstraints.minHeight + gridBagConstraints.centerPadding + n6) / 2 + gridBagConstraints.centerOffset != n4) {
                                --n6;
                            }
                            rectangle.height = gridBagConstraints.minHeight + n6;
                            rectangle.y = n + n4 - (rectangle.height + gridBagConstraints.centerPadding) / 2 - gridBagConstraints.centerOffset;
                            break;
                        }
                        case OTHER: {}
                    }
                }
            }
        }
        else {
            this.centerVertically(gridBagConstraints, rectangle, n2);
        }
    }
    
    private void alignAboveBaseline(GridBagConstraints gridBagConstraints, Rectangle rectangle, int n, int n2) {
        if (this.layoutInfo.hasBaseline(gridBagConstraints.tempY)) {
            int n3;
            if (this.layoutInfo.hasConstantDescent(gridBagConstraints.tempY)) {
                n3 = n + n2 - this.layoutInfo.maxDescent[gridBagConstraints.tempY];
            }
            else {
                n3 = n + this.layoutInfo.maxAscent[gridBagConstraints.tempY];
            }
            if (gridBagConstraints.isVerticallyResizable()) {
                rectangle.y = n + gridBagConstraints.insets.top;
                rectangle.height = n3 - rectangle.y;
            }
            else {
                rectangle.height = gridBagConstraints.minHeight + gridBagConstraints.ipady;
                rectangle.y = n3 - rectangle.height;
            }
        }
        else {
            this.centerVertically(gridBagConstraints, rectangle, n2);
        }
    }
    
    private void alignBelowBaseline(GridBagConstraints gridBagConstraints, Rectangle rectangle, int n, int n2) {
        if (this.layoutInfo.hasBaseline(gridBagConstraints.tempY)) {
            if (this.layoutInfo.hasConstantDescent(gridBagConstraints.tempY)) {
                rectangle.y = n + n2 - this.layoutInfo.maxDescent[gridBagConstraints.tempY];
            }
            else {
                rectangle.y = n + this.layoutInfo.maxAscent[gridBagConstraints.tempY];
            }
            if (gridBagConstraints.isVerticallyResizable()) {
                rectangle.height = n + n2 - rectangle.y - gridBagConstraints.insets.bottom;
            }
        }
        else {
            this.centerVertically(gridBagConstraints, rectangle, n2);
        }
    }
    
    private void centerVertically(GridBagConstraints gridBagConstraints, Rectangle rectangle, int n) {
        if (!gridBagConstraints.isVerticallyResizable()) {
            rectangle.y += Math.max(0, (n - gridBagConstraints.insets.top - gridBagConstraints.insets.bottom - gridBagConstraints.minHeight - gridBagConstraints.ipady) / 2);
        }
    }
    
    protected Dimension getMinSize(Container container, GridBagLayoutInfo gridBagLayoutInfo) {
        return this.GetMinSize(container, gridBagLayoutInfo);
    }
    
    protected Dimension GetMinSize(Container container, GridBagLayoutInfo gridBagLayoutInfo) {
        Dimension dimension = new Dimension();
        Insets insets = container.getInsets();
        int n = 0;
        for (int i = 0; i < gridBagLayoutInfo.width; ++i) {
            n += gridBagLayoutInfo.minWidth[i];
        }
        dimension.width = n + insets.left + insets.right;
        int n2 = 0;
        for (int j = 0; j < gridBagLayoutInfo.height; ++j) {
            n2 += gridBagLayoutInfo.minHeight[j];
        }
        dimension.height = n2 + insets.top + insets.bottom;
        return dimension;
    }
    
    protected void arrangeGrid(Container container) {
        this.ArrangeGrid(container);
    }
    
    protected void ArrangeGrid(Container container) {
        Insets insets = container.getInsets();
        Component[] components = container.getComponents();
        Rectangle rectangle = new Rectangle();
        this.rightToLeft = !container.getComponentOrientation().isLeftToRight();
        if (components.length == 0 && (this.columnWidths == null || this.columnWidths.length == 0) && (this.rowHeights == null || this.rowHeights.length == 0)) {
            return;
        }
        GridBagLayoutInfo layoutInfo = this.getLayoutInfo(container, 2);
        Dimension dimension = this.getMinSize(container, layoutInfo);
        if (container.width < dimension.width || container.height < dimension.height) {
            layoutInfo = this.getLayoutInfo(container, 1);
            dimension = this.getMinSize(container, layoutInfo);
        }
        this.layoutInfo = layoutInfo;
        rectangle.width = dimension.width;
        rectangle.height = dimension.height;
        int n = container.width - rectangle.width;
        int n2;
        if (n != 0) {
            double n3 = 0.0;
            for (int i = 0; i < layoutInfo.width; ++i) {
                n3 += layoutInfo.weightX[i];
            }
            if (n3 > 0.0) {
                for (int j = 0; j < layoutInfo.width; ++j) {
                    int n4 = (int)((double)n * layoutInfo.weightX[j] / n3);
                    int[] minWidth = layoutInfo.minWidth;
                    int n5 = j;
                    minWidth[n5] += n4;
                    rectangle.width += n4;
                    if (layoutInfo.minWidth[j] < 0) {
                        rectangle.width -= layoutInfo.minWidth[j];
                        layoutInfo.minWidth[j] = 0;
                    }
                }
            }
            n2 = container.width - rectangle.width;
        }
        else {
            n2 = 0;
        }
        int n6 = container.height - rectangle.height;
        int n7;
        if (n6 != 0) {
            double n8 = 0.0;
            for (int k = 0; k < layoutInfo.height; ++k) {
                n8 += layoutInfo.weightY[k];
            }
            if (n8 > 0.0) {
                for (int l = 0; l < layoutInfo.height; ++l) {
                    int n9 = (int)((double)n6 * layoutInfo.weightY[l] / n8);
                    int[] minHeight = layoutInfo.minHeight;
                    int n10 = l;
                    minHeight[n10] += n9;
                    rectangle.height += n9;
                    if (layoutInfo.minHeight[l] < 0) {
                        rectangle.height -= layoutInfo.minHeight[l];
                        layoutInfo.minHeight[l] = 0;
                    }
                }
            }
            n7 = container.height - rectangle.height;
        }
        else {
            n7 = 0;
        }
        layoutInfo.startx = n2 / 2 + insets.left;
        layoutInfo.starty = n7 / 2 + insets.top;
        for (int n11 = 0; n11 < components.length; ++n11) {
            Component componentAdjusting = components[n11];
            if (componentAdjusting.isVisible()) {
                GridBagConstraints lookupConstraints = this.lookupConstraints(componentAdjusting);
                if (!this.rightToLeft) {
                    rectangle.x = layoutInfo.startx;
                    for (int n13 = 0; n13 < lookupConstraints.tempX; ++n13) {
                        rectangle.x += layoutInfo.minWidth[n13];
                    }
                }
                else {
                    rectangle.x = container.width - (n2 / 2 + insets.right);
                    for (int n12 = 0; n12 < lookupConstraints.tempX; ++n12) {
                        rectangle.x -= layoutInfo.minWidth[n12];
                    }
                }
                rectangle.y = layoutInfo.starty;
                for (int n14 = 0; n14 < lookupConstraints.tempY; ++n14) {
                    rectangle.y += layoutInfo.minHeight[n14];
                }
                rectangle.width = 0;
                for (int tempX = lookupConstraints.tempX; tempX < lookupConstraints.tempX + lookupConstraints.tempWidth; ++tempX) {
                    rectangle.width += layoutInfo.minWidth[tempX];
                }
                rectangle.height = 0;
                for (int tempY = lookupConstraints.tempY; tempY < lookupConstraints.tempY + lookupConstraints.tempHeight; ++tempY) {
                    rectangle.height += layoutInfo.minHeight[tempY];
                }
                this.componentAdjusting = componentAdjusting;
                this.adjustForGravity(lookupConstraints, rectangle);
                if (rectangle.x < 0) {
                    rectangle.width += rectangle.x;
                    rectangle.x = 0;
                }
                if (rectangle.y < 0) {
                    rectangle.height += rectangle.y;
                    rectangle.y = 0;
                }
                if (rectangle.width <= 0 || rectangle.height <= 0) {
                    componentAdjusting.setBounds(0, 0, 0, 0);
                }
                else if (componentAdjusting.x != rectangle.x || componentAdjusting.y != rectangle.y || componentAdjusting.width != rectangle.width || componentAdjusting.height != rectangle.height) {
                    componentAdjusting.setBounds(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
                }
            }
        }
        return;
    }
}
